<?php

/**
 * @file
 * Contains \Drupal\ces_import\ImportBase.
 *
 */
namespace Drupal\ces_import;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use \Drupal\Core\Form\FormBase;

ini_set('max_execution_time', 60);//this is for the validation of large sets

abstract class ImportBase extends FormBase {

  private $database;

  /**
   * @var \GuzzleHttp
   */
  private $client;

  protected $entityTypeManager;
  protected $tempStore;

  /**
   * @todo injections
   */
  function __construct($database, $http_client, $entity_type_manager, $user_tempstore) {
    $this->database = $database;
    $this->client = $http_client;
    $this->entityTypeManager = $entity_type_manager;
    $this->tempStore = $user_tempstore->get('tempImport');
  }

  static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('http_client'),
      $container->get('entity_type.manager'),
      $container->get('user.private_tempstore')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['which'] = [
      '#title' => $this->t('Data source'),
      '#type' => 'radios',
      '#options' => [
        'inline' => $this->t('Inline'),
        'url' => $this->t('remote')
      ],
      '#default_value' => 'inline',
      '#required' => TRUE,
      '#weight' => 2
    ];

    $form['delete'] = array(
      '#title' => t('Delete existing'),
      '#type' => 'checkbox',
      '#default_value' => TRUE,
      '#weight' => 2
    );

    $form['data'] = array (
      '#title' => 'paste data',
      '#type' => 'textarea',
      '#default_value' => $form_state->getValue('data'),
      '#rows' => 15,
      '#weight' => 1,
      '#states' => [
        'visible' => [
          ':input[name="which"]' => ['value' => 'inline']
        ]
      ],
      '#weight' => 3
    );
    $placeholder = '';
    foreach ($this->columns() as $col) {
      $placeholder[] = '"'.$col.'"';
    }
    $form['data']['#placeholder'] = implode(', ',  $placeholder);

    $form['url'] = [
      '#title' => $this->t('Url'),
      '#type' => 'url',
      '#default_value' => $form_state->getValue('url'),
      '#states' => [
        'visible' => [
          ':input[name="which"]' => ['value' => 'url']
        ]
      ],
      '#weight' => 3
    ];
    $form['operation'] = array(
      '#title' => t('Validate'),
      '#type' => 'radios',
      '#options' => [
        'both' => $this->t('Normal'),
        'val' => $this->t('Validate only'),
        'sub' => $this->t('Submit only')
      ],
      '#default_value' => 'val',
      '#weight' => 4
    );
    $form['just2'] = array(
      '#title' => t('Only process 2'),
      '#type' => 'checkbox',
      '#default_value' => TRUE,
      '#weight' => 6
    );
    $form['submit'] = array (
      '#type' => 'submit',
      '#value' => 'import',
      '#weight' => 7
    );
    $form['#redirect'] = \Drupal\Core\Url::fromRoute('entity.smallad.collection');
    return $form;
  }

  public function validateForm(array &$form, FormStateInterface $form_state) {
    if($form_state->getvalue('operation') == 'sub') {
      return;
    }
    $data = $this->getData($form_state);

    foreach ($this->getCsvRows($data) as $rownum => $row) {
      try {
        $entity = $this->buildEntity($row);
      }
      catch (\Exception $e) {
        drupal_set_message('',$e->getMessage() . ' on row '.implode(', ', $row), 'error');
        continue;
      }
      if (!drupal_get_messages('error')) {
        foreach ($entity->validate() as $violation) {
          drupal_set_message($violation->getMessage().' at '.$violation->getpropertyPath() .print_r(implode(', ', $row), 1), 'error');
        }
      }
    }
  }

  function getData($form_state) {
    if ($form_state->getValue('which') == 'inline' ) {
      $data = $form_state->getValue('data');
    }
    else {
      $data = $this->client->get($form_state->getValue('url'))->getBody()->getContents();
      if (!$data) {
        throw new \Exception('no data or password wrong');
      }
    }
    return $data;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    if ($form_state->getvalue('operation') == 'val') {
      return;
    }
    $data = $this->getData($form_state);
    $rows = $this->getCsvRows($data);
    if ($form_state->getvalue('just2')) {
      $rows = array_splice($rows, 0, 2);
    }
    $batch = $this->makeBatch($rows, $form_state->getValue('delete'));
    batch_set($batch);
  }

  function makeBatch($rows, $delete = FALSE) {
    $batch = ['title' => 'Creating entities'];//this could be overwritten
    if ($delete) {
      $batch['operations'][] = [[$this, 'deleteAll'], []];
    }
    foreach (array_chunk($rows, 50) as $chunk) {
      $batch['operations'][] = [[$this, 'saveEntities'], [$chunk]];
    }
    return $batch;
  }

  function getCsvRows($data) {
    //$data = html_entity_decode($data);
    $csv = array_map('str_getcsv', explode(PHP_EOL, trim($data)));

    array_walk($csv, function(&$a) use ($csv) {
      $a = array_combine($csv[0], $a);
    });
    array_shift($csv); # remove first row
    $output = [];
    foreach ($csv as $key => $row) {
      foreach ($this->columns() as $col) {
        $output[$key][$col] = $row[$col];
      }
    }
    return $output;
  }

  function buildEntity($fields) {
    $entity = $this->createEntity();
    foreach ($fields as $fieldname => $value) {
      if (strpos($fieldname, '.')) {
        list($field, $col) = explode('.', $fieldname);
        $process_function = $field.'_'.$col.'Process';
        if (method_exists($this, $method)) {
         $this->{$process_function}($entity, $fields[$fieldname], $fields);
        }
        else {
          $entity->{$field}->{$col} = $value;
        }
      }
      elseif(in_array($fieldname, ['created', 'changed', 'access', 'expires'])) {
        $this->timeProcess($entity, $fieldname, $fields[$fieldname]);
      }
      else {
        $method = $fieldname.'Process';
        if (method_exists($this, $method)) {
          $this->{$method}($entity, $fields[$fieldname], $fields);
        }
        else {
          $entity->set($fieldname, $value);
        }
      }
    }
    return $entity;
  }

  protected function createEntity() {
    $entityType = \Drupal::entityTypeManager()->getDefinition($this->entityTypeId);
    $key = $entityType->getKey('bundle') ? : $this->entityTypeId;
    return \Drupal::entityTypeManager()
      ->getStorage($this->entityTypeId)
      ->create([$key => $this->bundle]);
  }


  //batch callback.
  //the sandbox seems to be empty every time, unfortunately
  function saveEntities($rows, &$sandbox) {
    foreach ($rows as $rownum => $row) {
      $this->buildEntity($row)->save();
    }
  }


  function timeProcess(EntityInterface $account, $fieldname, $val) {
    $account->set($fieldname, strtotime($val));
  }

  function langcodeProcess(EntityInterface $account, $val, $allfields) {
    $account->set('langcode', $val);
    $account->set('preferred_langcode', $val);
  }

  function required(&$val, $default = NULL) {
    if (empty($val)) {
      if ($default) {
        $val = $default;
      }
      else {
        throw new \Exception("$fieldname cannot be empty");
      }
    }
  }

}
